%% This script demonstrates the reduction in over-fitting as the 
 % regularization coefficient is increased. A polynomial is fit using 
 % ML estimation using lambda=0 (no regularization), exp(-18), exp(-6)
 % and 1.
 % No regularization (lambda=0) leads to over-fitting while high
 % regularization (lambda=1) leads to poor fitting.
clf;
load data_10; 
M=9;
newx=0:.01:1;

lambda=0;
[w0,beta,E_RMS_train0]=fit_polynomial_ML(x,t,M,lambda);
subplot(221);
plot_fitting_results(newx,w0,x,t);
title(['E_{RMS}=' num2str(E_RMS_train0)]);
text(.2,-1.3,['$\lambda=$' num2str(lambda)],'Interpreter','Latex');

lambda=exp(-18);
[we_18,beta,E_RMS_traine_18]=fit_polynomial_ML(x,t,M,lambda);
subplot(222);
plot_fitting_results(newx,we_18,x,t);
title(['E_{RMS}=' num2str(E_RMS_traine_18)]);
text(.2,-1.3,['$\lambda=e^{-18}$'],'Interpreter','Latex');

lambda=exp(-6);
[we_6,beta,E_RMS_traine_6]=fit_polynomial_ML(x,t,M,lambda);
subplot(223);
plot_fitting_results(newx,we_6,x,t);
title(['E_{RMS}=' num2str(E_RMS_traine_6)]);
text(.2,-1.3,['$\lambda=e^{-6}$'],'Interpreter','Latex');

lambda=1;
[w1,beta,E_RMS_train1]=fit_polynomial_ML(x,t,M,lambda);
subplot(224);
plot_fitting_results(newx,w1,x,t);
title(['E_{RMS}=' num2str(E_RMS_train1)]);
text(.2,-1.3,['$\lambda=$' num2str(lambda)],'Interpreter','Latex');

print(gcf,'Figure_1_7','-dpng');

%% fitted polynomial coefficients (similar to Bishop, Table 1.2)
 % w0 corresponding to lambda=0 has large magnitudes which is a sign of
 % over-fitting
coefficients_regularization=[w0 we_18 we_6 w1];
disp(vpa(coefficients_regularization,5));
save coefficients_regularization.mat coefficients_regularization